// For a detailed explanation regarding each configuration property, visit:
// https://jestjs.io/docs/en/configuration.html

module.exports = {
    moduleDirectories: [
        "src",
        "node_modules",
    ],
    testMatch: [
        "<rootDir>/test/*.js",
    ]
};
